"""
termcolorx.logger — Logger berwarna dengan opsi simpan ke file.
"""

from datetime import datetime
from .color import TermColorX

class LoggerColor:
    COLORS = {
        "INFO": ("green", None), "WARNING": ("yellow", None),
        "ERROR": ("red", None), "DEBUG": ("cyan", None),
        "SUCCESS": ("blue", None), "CRITICAL": ("white", "red")
    }

    def __init__(self, name="LoggerColor", log_file=None):
        self.name = name
        self.log_file = log_file

    def _format_message(self, level, message):
        fg, bg = self.COLORS.get(level, ("white", None))
        timestamp = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        level_text = TermColorX.colorize(f"[{level}]", fg=fg, bg=bg, style="bold")
        plain = f"{timestamp} [{level}] {message}"
        colored = f"{timestamp} {level_text} {message}"
        return plain, colored

    def log(self, level, message):
        plain, colored = self._format_message(level.upper(), message)
        print(colored)
        if self.log_file:
            with open(self.log_file, "a", encoding="utf-8") as f:
                f.write(plain + "\n")

    def info(self, msg): self.log("INFO", msg)
    def warning(self, msg): self.log("WARNING", msg)
    def error(self, msg): self.log("ERROR", msg)
    def debug(self, msg): self.log("DEBUG", msg)
    def success(self, msg): self.log("SUCCESS", msg)
    def critical(self, msg): self.log("CRITICAL", msg)



"""
termcolorx.color — Pewarnaan teks terminal dasar, 256, dan RGB.
"""

class TermColorX:
    COLORS = {
        "black": 30, "red": 31, "green": 32, "yellow": 33,
        "blue": 34, "magenta": 35, "cyan": 36, "white": 37, "reset": 39
    }
    BACKGROUNDS = {
        "black": 40, "red": 41, "green": 42, "yellow": 43,
        "blue": 44, "magenta": 45, "cyan": 46, "white": 47, "reset": 49
    }
    STYLES = {
        "bold": 1, "dim": 2, "italic": 3, "underline": 4,
        "blink": 5, "reverse": 7, "hidden": 8, "reset": 0
    }

    @staticmethod
    def colorize(text, fg=None, bg=None, style=None):
        codes = []
        if fg and fg in TermColorX.COLORS: codes.append(str(TermColorX.COLORS[fg]))
        if bg and bg in TermColorX.BACKGROUNDS: codes.append(str(TermColorX.BACKGROUNDS[bg]))
        if style and style in TermColorX.STYLES: codes.append(str(TermColorX.STYLES[style]))
        prefix = f"\033[{';'.join(codes)}m" if codes else ""
        return f"{prefix}{text}\033[0m"

    @staticmethod
    def rgb(text, r, g, b, background=False):
        return f"\033[{48 if background else 38};2;{r};{g};{b}m{text}\033[0m"

    @staticmethod
    def color256(text, code, background=False):
        return f"\033[{48 if background else 38};5;{code}m{text}\033[0m"

    # Shortcut
    @staticmethod
    def red(t): return TermColorX.colorize(t, fg="red")
    @staticmethod
    def green(t): return TermColorX.colorize(t, fg="green")
    @staticmethod
    def yellow(t): return TermColorX.colorize(t, fg="yellow")
    @staticmethod
    def blue(t): return TermColorX.colorize(t, fg="blue")
    @staticmethod
    def magenta(t): return TermColorX.colorize(t, fg="magenta")
    @staticmethod
    def cyan(t): return TermColorX.colorize(t, fg="cyan")
    @staticmethod
    def bold(t): return TermColorX.colorize(t, style="bold")
    @staticmethod
    def underline(t): return TermColorX.colorize(t, style="underline")
    @staticmethod
    def blink(t): return TermColorX.colorize(t, style="blink")
    @staticmethod
    def reverse(t): return TermColorX.colorize(t, style="reverse")
    @staticmethod
    def dim(t): return TermColorX.colorize(t, style="dim")
    @staticmethod
    def italic(t): return TermColorX.colorize(t, style="italic")
    @staticmethod
    def hidden(t): return TermColorX.colorize(t, style="hidden")


from .color import TermColorX
from .logger import LoggerColor
from .gradient import Gradient
from .animation import Animation
from .platform import PlatformSupport

__all__ = ["TermColorX", "LoggerColor", "Gradient", "Animation", "PlatformSupport"]



"""
termcolorx.animation — Efek animasi teks terminal (spinner, typing, dll).
"""

import sys, time
import asyncio
from .color import TermColorX

class Animation:
    @staticmethod
    def typing(text, delay=0.05, color="cyan"):
        for ch in text:
            sys.stdout.write(TermColorX.colorize(ch, fg=color))
            sys.stdout.flush()
            time.sleep(delay)
        print()

    @staticmethod
    def spinner(duration=3, color="yellow"):
        spinner_chars = ['|', '/', '-', '\\']
        end_time = time.time() + duration
        i = 0
        while time.time() < end_time:
            sys.stdout.write("\r" + TermColorX.colorize(spinner_chars[i % 4], fg=color))
            sys.stdout.flush()
            time.sleep(0.1)
            i += 1
        sys.stdout.write("\r")
        print(TermColorX.green("✔ Done!"))
    @staticmethod
    def loading_dots(count=3, delay=0.5, color="magenta"):
        for i in range(count):
            sys.stdout.write(TermColorX.colorize(".", fg=color))
            sys.stdout.flush()
            time.sleep(delay)
        print()
    @staticmethod
    def progress_bar(total=30, duration=5, color="green"):
        for i in range(total + 1):
            percent = (i / total) * 100
            bar = '█' * i + '░' * (total - i)
            sys.stdout.write(f"\r{TermColorX.colorize(f'[{bar}] {percent:.2f}%', fg=color)}")
            sys.stdout.flush()
            time.sleep(duration / total)
        print()
    @staticmethod
    def clear_line():
        sys.stdout.write("\r" + " " * 80 + "\r")
        sys.stdout.flush()
    @staticmethod
    def type_and_spinner(text, typing_delay=0.05, spinner_duration=3, color="cyan"):
        Animation.typing(text, delay=typing_delay, color=color)
        Animation.spinner(duration=spinner_duration, color=color)
    @staticmethod
    def loading_message(message, dots=3, dot_delay=0.5, color="magenta"):
        sys.stdout.write(TermColorX.colorize(message, fg=color))
        sys.stdout.flush()
        Animation.loading_dots(count=dots, delay=dot_delay, color=color)
    @staticmethod
    def progress_with_message(message, total=30, duration=5, color="green"):
        sys.stdout.write(TermColorX.colorize(message + " ", fg=color))
        sys.stdout.flush()
        Animation.progress_bar(total=total, duration=duration, color=color)
    @staticmethod
    async def async_spinner(message, duration=3, color="yellow"):
        sys.stdout.write(TermColorX.colorize(message + " ", fg=color))
        sys.stdout.flush()
        spinner_chars = ['|', '/', '-', '\\']
        end_time = time.time() + duration
        i = 0
        while time.time() < end_time:
            sys.stdout.write("\r" + TermColorX.colorize(message + " " + spinner_chars[i % 4], fg=color))
            sys.stdout.flush()
            await asyncio.sleep(0.1)
            i += 1
        sys.stdout.write("\r")
        print(TermColorX.green("✔ Done!"))
    @staticmethod
    async def async_type_and_spinner(text, typing_delay=0.05, spinner_duration=3, color="cyan"):
        for ch in text:
            sys.stdout.write(TermColorX.colorize(ch, fg=color))
            sys.stdout.flush()
            await asyncio.sleep(typing_delay)
        print()
        await Animation.async_spinner("Processing...", duration=spinner_duration, color=color)
    @staticmethod
    async def async_progress_with_message(message, total=30, duration=5, color="green"):
        sys.stdout.write(TermColorX.colorize(message + " ", fg=color))
        sys.stdout.flush()
        for i in range(total + 1):
            percent = (i / total) * 100
            bar = '█' * i + '░' * (total - i)
            sys.stdout.write(f"\r{TermColorX.colorize(message + ' [' + bar + f'] {percent:.2f}%', fg=color)}")
            sys.stdout.flush()
            await asyncio.sleep(duration / total)
        print()
    @staticmethod
    def multiline_typing(lines, delay=0.05, color="cyan"):
        for line in lines:
            Animation.typing(line, delay=delay, color=color)
            time.sleep(0.3)
    @staticmethod
    async def async_multiline_typing(lines, delay=0.05, color="cyan"):
        for line in lines:
            for ch in line:
                sys.stdout.write(TermColorX.colorize(ch, fg=color))
                sys.stdout.flush()
                await asyncio.sleep(delay)
            print()
            await asyncio.sleep(0.3)


"""
termcolorx.platform — Deteksi otomatis dukungan warna terminal.
"""

import os, sys, platform

class PlatformSupport:
    @staticmethod
    def enable_ansi():
        if os.name == 'nt':
            os.system("")  # Enable ANSI di Windows 10+
        return True

    @staticmethod
    def is_supported():
        if sys.stdout.isatty():
            return True
        return False

    @staticmethod
    def info():
        return {
            "os": platform.system(),
            "version": platform.version(),
            "ansi_supported": PlatformSupport.is_supported()
        }
    



"""
termcolorx.gradient — Membuat teks gradien warna RGB.
"""

from .color import TermColorX
import asyncio

class Gradient:
    @staticmethod
    def rainbow(text):
        colors = [
            (255, 0, 0), (255, 127, 0), (255, 255, 0),
            (0, 255, 0), (0, 0, 255), (75, 0, 130), (148, 0, 211)
        ]
        result = ""
        for i, ch in enumerate(text):
            r, g, b = colors[i % len(colors)]
            result += TermColorX.rgb(ch, r, g, b)
        return result
    @staticmethod
    async def rainbow_dynamic(text, delay=0.1, cycles=3):
        colors = [
            (255, 0, 0), (255, 127, 0), (255, 255, 0),
            (0, 255, 0), (0, 0, 255), (75, 0, 130), (148, 0, 211)
        ]
        length = len(colors)
        for cycle in range(cycles):
            result = ""
            for i, ch in enumerate(text):
                r, g, b = colors[(i + cycle) % length]
                result += TermColorX.rgb(ch, r, g, b)
            print(f"\r{result}", end="", flush=True)
            await asyncio.sleep(delay)
        print()
    @staticmethod
    def custom_gradient(text, start_color, end_color):
        def interpolate(c1, c2, factor):
            return int(c1 + (c2 - c1) * factor)
        result = ""
        length = len(text)
        for i, ch in enumerate(text):
            factor = i / max(length - 1, 1)
            r = interpolate(start_color[0], end_color[0], factor)
            g = interpolate(start_color[1], end_color[1], factor)
            b = interpolate(start_color[2], end_color[2], factor)
            result += TermColorX.rgb(ch, r, g, b)
        return result
    @staticmethod
    def vertical_gradient(text, start_color, end_color):
        lines = text.splitlines()
        result = ""
        length = len(lines)
        def interpolate(c1, c2, factor):
            return int(c1 + (c2 - c1) * factor)
        for i, line in enumerate(lines):
            factor = i / max(length - 1, 1)
            r = interpolate(start_color[0], end_color[0], factor)
            g = interpolate(start_color[1], end_color[1], factor)
            b = interpolate(start_color[2], end_color[2], factor)
            colored_line = TermColorX.rgb(line, r, g, b)
            result += colored_line + "\n"
        return result.rstrip("\n")
    @staticmethod
    async def vertical_gradient_dynamic(text, start_color, end_color, delay=0.2, cycles=3):
        lines = text.splitlines()
        length = len(lines)
        def interpolate(c1, c2, factor):
            return int(c1 + (c2 - c1) * factor)
        for cycle in range(cycles):
            result = ""
            for i, line in enumerate(lines):
                factor = (i + cycle) / max(length - 1 + cycles, 1)
                r = interpolate(start_color[0], end_color[0], factor)
                g = interpolate(start_color[1], end_color[1], factor)
                b = interpolate(start_color[2], end_color[2], factor)
                colored_line = TermColorX.rgb(line, r, g, b)
                result += colored_line + "\n"
            print(f"\r{result}", end="", flush=True)
            await asyncio.sleep(delay)
        print()
    @staticmethod
    def horizontal_gradient(text, start_color, end_color):
        result = ""
        length = len(text)
        def interpolate(c1, c2, factor):
            return int(c1 + (c2 - c1) * factor)
        for i, ch in enumerate(text):
            factor = i / max(length - 1, 1)
            r = interpolate(start_color[0], end_color[0], factor)
            g = interpolate(start_color[1], end_color[1], factor)
            b = interpolate(start_color[2], end_color[2], factor)
            result += TermColorX.rgb(ch, r, g, b)
        return result
    @staticmethod
    async def horizontal_gradient_dynamic(text, start_color, end_color, delay=0.1, cycles=3):
        length = len(text)
        def interpolate(c1, c2, factor):
            return int(c1 + (c2 - c1) * factor)
        for cycle in range(cycles):
            result = ""
            for i, ch in enumerate(text):
                factor = (i + cycle) / max(length - 1 + cycles, 1)
                r = interpolate(start_color[0], end_color[0], factor)
                g = interpolate(start_color[1], end_color[1], factor)
                b = interpolate(start_color[2], end_color[2], factor)
                result += TermColorX.rgb(ch, r, g, b)
            print(f"\r{result}", end="", flush=True)
            await asyncio.sleep(delay)
        print()
    


from setuptools import setup, find_packages

setup(
    name="termcolorx",
    version="0.0.1",
    author="ATHALLAH RAJENDRA PUTRA JUNIARTO",
    author_email="athallahwork50@gmail.com",
    description="Library pewarna teks terminal lengkap (ANSI, RGB, Gradien, Logger, Animasi) dari dasar.",
    long_description=open("README.md", encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/Athallah1234/termcolorx",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Topic :: Terminals",
        "Topic :: Utilities",
        "Operating System :: OS Independent"
    ],
    python_requires=">=3.13",
)
